var express = require('express')
  , expressNamespace = require('express-namespace')
  , fs = require('fs')
  , vm = require('vm')
  , stat = fs.statSync
  , ls = fs.readdirSync
  , path = require('path')
  , util = require('util')
  , inspect = util.inspect
  , join = path.join

  , fructose = require('fructose')

//var utils = require('./utils.js')
//  , requireVersion = '0.5.2'

//if(!utils.isNodeVersion(requireVersion))
//  throw new Error('Node >=' + requireVersion + ' required')

var exedra = function(variables) {
  variables.inspect = inspect
  var _variables = fructose.merge({}, variables, global)
  
  var cwd = process.cwd()
    , app = _variables.app
    , fn = {}

  /* ->
   * Все, что относится к загрузке функций
   */
  var loadRecursive = function(dir, link){
    var result = link || {}

    ls(dir).forEach(function(name) {
      if(fructose.notTempOrHidden(name)) {
        var path = join(dir, name)
          , key = path.replace(dir, '').replace(/^\//, '').replace(/\.js$/i, '')

        if(!result[key])
          result[key] = {}
        
        if(stat(path).isDirectory()) {
          loadRecursive(path, result[key])
        } else {
          var module = require(path)
          if(module instanceof Function) {
            var context = vm.createContext(_variables)
              , runtime = vm.runInContext('(' + module.toString() + ')()', context)
              
            result[key] = fructose.merge(result[key], runtime)
          }
        }
      }
    })

    return result
  }
  
  fn.functions = function(){
    var dir = app.set('path.functions') // . это просто разделитель в строке, к объектам отношения не имеет  
    if(!dir)
      throw new Error('App "path.functions" setting is undefined')
    
    _variables.fn = loadRecursive(dir)
    
    return this
  }
  
  /* <-
   * Все, что относится к загрузке функций
   */


  /* ->
   * Загрузка маршрутов.
   * ! Должна быть вызвана последней !
   */
  fn.routes = function(){
    var dir = app.set('path.routes') // . это просто разделитель в строке, к объектам отношения не имеет  
    if(!dir)
      throw new Error('App "path.routes" setting is undefined')
    
    var namespaces = {}
      
    fructose.lsRecursive(dir).forEach(function(route) {
      route = route.replace(dir, '').replace(/\.js$/i, '')
      
      var path = join(dir, route)
      
      route = route.replace(/index$/i, '').replace(/\/index/, '/')
      
      if(!namespaces[route])
        namespaces[route] = []
      
      namespaces[route].push({
        route: route,
        init: require(path),
        path: path
      })
      
    })
    
    namespaces = fructose.toArray(namespaces)
    namespaces.forEach(function(routes){
      app.namespace(routes[0].route, function(){
        var self = this
          , context = vm.createContext(fructose.merge({ namespace: self }, _variables))
        routes.forEach(function(route){
          if(typeof route.init == 'function')
            vm.runInContext('(' + route.init.toString() + ').apply(namespace)', context)
          else
            console.error('  \033[31;2m%s\033[0m exports not function', route.path)
        })
      })
    })
    
    return this
  }
  
  return fn;

}
module.exports = exedra
